/*
This is the code for the AirGradient DIY Air Quality Sensor with an ESP8266 Microcontroller.

It is a high quality sensor showing PM2.5, CO2, Temperature and Humidity on a small display and can send data over Wifi.

For build instructions please visit https://www.airgradient.com/diy/

Compatible with the following sensors:
Plantower PMS5003 (Fine Particle Sensor)
SenseAir S8 (CO2 Sensor)
SHT30/31 (Temperature/Humidity Sensor)

Dependent Libraries:
The codes needs the following libraries installed:
ESP8266 board with standard libraries
WifiManager by tzar, tablatronix tested with Version 2.0.3-alpha
ESP8266 and ESP32 OLED driver for SSD1306 displays by ThingPulse, Fabrice Weinberg tested with Version 4.1.0

Configuration:
Please set in the code below which sensor you are using and if you want to connect it to WiFi.

If you are a school or university contact us for a free trial on the AirGradient platform.
https://www.airgradient.com/schools/

MIT License
*/

#include <Wire.h>
#include <AirGradient.h>
//#include <WiFiManager.h>
//#include <ESP8266WiFi.h>
//#include <ESP8266HTTPClient.h>

#include "SSD1306Wire.h"

AirGradient ag = AirGradient();

SSD1306Wire display(0x3c, SDA, SCL);

// set sensors that you do not use to false
boolean hasPM=true;
boolean hasCO2=true;
boolean hasSHT=true;

// set to true if you want to connect to wifi. The display will show values only when the sensor has wifi connection
boolean connectWIFI=false;

// change if you want to send the data to another server
String APIROOT = "http://hw.airgradient.com/";


// DISPLAY
void showTextAll(String ln1, String ln2, String ln3) {
  display.clear();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_16);
  display.drawString(32, 16, ln1);
  display.drawString(32, 32, ln2);
  display.drawString(32, 48, ln3);
  display.display();
}

void setup(){
  display.init();
  display.flipScreenVertically();
  showTextAll("Init", String(ESP.getChipId(),HEX), "");

  if (hasPM) ag.PMS_Init();
  if (hasCO2) ag.CO2_Init();
  if (hasSHT) ag.TMP_RH_Init(0x44);
}

void loop(){
  if (hasPM && hasCO2 && hasSHT) {
    int PM2 = ag.getPM2_Raw();
    int CO2 = ag.getCO2_Raw();
    TMP_RH result = ag.periodicFetchData();
    showTextAll("pm²⁵ " + String(PM2), "co² " + String(CO2), String((int)result.t)+"°/"+ String(result.rh)+"%");
    delay(3000);
  }
}
